/*
  Copyright (c) 2015, Matthias Schiffer <mschiffer@universe-factory.net>
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#pragma once

#include <string>
#include <unordered_set>
#include <unordered_map>
#include <utility>
#include <vector>


namespace solar {

class state_t {
private:
	struct sym_t {
		virtual ~sym_t() {}
	};

	struct nonterm_sym_t : public sym_t {
		std::string value;

		nonterm_sym_t(const std::string &value0) : value(value0) {}
	};

	struct term_sym_t : public sym_t {
		std::string value;

		term_sym_t(const std::string &value0) : value(value0) {}
	};

	struct char_sym_t : public sym_t {
		unsigned char value;

		char_sym_t(unsigned char value0) : value(value0) {}
	};


	typedef std::vector<sym_t> rhs_t;

	std::unordered_set<std::string> terminals;
	std::unordered_multimap<std::string, rhs_t> rules;

	std::string current_nonterm;
	rhs_t current_rule;

public:
	void openRule(const std::string &nonterm);
	void addRuleTerminal(const std::string &term);
	void addRuleTerminal(unsigned char term);
	void addRuleNonterminal(const std::string &nonterm);
	void closeRule();
};

}
